# GrapeCity Documents PDF Viewer

#### [日本語](#japanese)

A full-featured JavaScript PDF viewer and editor that comes with [GrapeCity Documents for PDF](https://www.grapecity.com/documents-api-pdf).

__GrapeCity Documents PDF Viewer__ (__GcDocs PDF Viewer__, __GcPdfViewer__) is a fast, modern JavaScript based PDF viewer and editor that runs in all major browsers.
The viewer can be used as a cross platform solution to view (or modify - see _Support API_ below) PDF documents on Windows, MAC, Linux, iOS and Android devices.
GcPdfViewer is included in [GrapeCity Documents for PDF (GcPdf)](https://www.grapecity.com/documents-api-pdf) - a feature-rich cross-platform PDF API library for .NET Core.

[_Support API_](#support_api) is a server-side NuGet package
[GrapeCity.Documents.Pdf.ViewerSupportApi](https://www.nuget.org/packages/GrapeCity.Documents.Pdf.ViewerSupportApi/)
that allows you to easily set up an ASP.&#8203;NET Core or a Web Forms server that employs GcPdf
to add PDF editing abilities to GcPdfViewer.
When connected to a _Support API_ server, GcPdfViewer becomes a powerful PDF editor that can be used
to edit existing or create new PDF documents, fill and save PDF forms,
remove (redact) sensitive content, add or edit annotations and AcroForm fields, and more.

GcPdfViewer provides a rich client side JavaScript object model, see __docs/index.html__ for the client API documentation.

Product highlights:

- Works in all modern browsers, including Edge, Chrome, FireFox, Opera, Safari
- When connected to _GcPdf_ on the server via _Support API_, provides:
  - Customizable and mobile-friendly form filler
  - Real-time collaboration mode
  - Annotation and form editors
  - Quick edits using secondary toolbars
  - PDF redaction
  - Signature verification
  - Other editing features
- Works with frameworks such as React, Preact, Angular
- Supports form filling; filled forms can be printed or form data can be submitted to the server
- Supports XFA (XML Forms Architecture) forms
- Provides caret for text selection/copy, including vertical and RTL texts
- Includes thumbnails, text search, outline, attachments, articles, layers and structure tags panels
- Allows opening PDF files from local disks
- Supports annotations including text, free text, rich text etc.
- Supports redact annotations (including appearance streams), allows user to hide or show redacts
- Supports sound annotations
- Allows rotating and printing the rotated document
- Supports article threads navigation
- Fully supports file attachments (both attachment annotations and document level attachments)
- Comes with several themes, new custom themes can be added
- Supports external CMaps
- ...and more.

## See it in action

- Go to [GrapeCity Documents for PDF Viewer demos](https://www.grapecity.com/documents-api-pdfviewer/demos/)
  to explore the various features of GcPdfViewer, including features that rely on [_Support API_](#support_api).
  The demo site allows you to modify the demo code and immediately see the effect of your changes.
- The [GrapeCity Documents for PDF API demo site](https://www.grapecity.com/documents-api-pdf/demos/)
  uses GcPdfViewer to display sample PDFs.

## Latest changes

## [4.1.2] - 08-May-2023
### Added
- [Form editor] Added tooltip support for all field widgets, added tooltip property editor. (DOC-5384)
- Added addStamp method. (DOC-5376)
```javascript
// Example: add graphical signature to the PDF using external image:
function addStampFromUrl(imageUrl, viewer) {
 fetch(imageUrl)
 .then(response => response.blob())
 .then(blob => blob.arrayBuffer())
 .then(arrayBuffer => {
       const fileId = new Date().getTime() + ".png";
       const fileName = fileId;
       const pageIndex = 0;
       const imageData = new Uint8Array(arrayBuffer);
       const rect = [0, 0, 200, 200];
       viewer.storage.setItem(fileId, imageData);
       viewer.addStamp(
                   imageData,
                   {
                       fileId,
                       fileName,
                       pageIndex,
                       rect,
                       select: false,
                       subject: "",
                       rotate: 0,
                       convertToContent: false
                   });     
 });
}
addStampFromUrl("http://example.com/image.png", viewer);
```
- Added jsExecutionConfig option: optional execution configuration for JavaScript actions.
```javascript
// Example 1: adjust jsCode before execution:
viewer.options.jsExecutionConfig = {
  before: function() {
      args.jsCode = args.jsCode.replace("app.alert", "app.showMessage");
  }
}
// Example 2: cancel JS execution:
viewer.options.jsExecutionConfig = {
  before: function(args) {
      args.cancel = true;
  }
}
// Example 3: execute long operation before JS execution and repaint visible pages after JS execution:
viewer.options.jsExecutionConfig = {
  before: function() {
      // Returned promise will be awaited before further JS execution.
      return new Promise(function(resolve) {
	         setTimeout(resolve, 1000);
      });
  },
  after: function() {
      viewer.repaint();
  }
}
```
### Changed
- [Editor] Updated property labels for field widgets: property "Name" renamed to "Field name", property "Value" renamed to "Field value", property "Export Value" renamed to "Choice value".
### Fixed
- The second toolbar position is not updated in some cases. (DOC-5377)
- [iOS] When a signature is drawn by Apple Pencil, a lens is sometimes displayed. (DOC-5370)
- [iOS] Locking all fields does not lock some properties. (DOC-5299)
- [iOS] Several issues when using the PDF Organizer. (DOC-5344, DOC-5345)
- Stamp annotations in some PDFs incorrectly appear rotated. (DOC-5383)
- Cannot copy a layer name from the Layers panel. (DOC-5385)
- Cannot load a stamp generated from an image. (DOC-5379)
- JavaScript actions' sequence is incorrect for focus/blur events. (DOC-5389)
- Localization issues. (DOC-5359)
- [Editor] Incorrect popup annotation orientation is some cases. (DOC-5386)

## [4.1.1] - 17-Apr-2023
### Fixed
- [Regression] [Editor] 'Support API server version 6.1.0 is out of date' warning incorrectly shows when the viewer is connected to SupportApi v6.1.0. (DOC-5374)

## [4.1.0] - 03-Apr-2023
### Added
- PDF Organizer: new feature that allows users to rearrange, duplicate or remove pages of a PDF, or merge PDFs. (DOC-3914, DOC-3915, DOC-3916)\
  To open the PDF Organizer dialog, in the top toolbar click 'Page Tools', and in the secondary toolbar click 'PDF Organizer'.
- Respect PDF's initial view settings (hide toolbars or menus, open with specific page layout, etc). (DOC-4667)
- Added ignoreInitialView option: set this option to true to ignore initial view settings specified in PDFs.
```javascript
// Example:
viewer.options.ignoreInitialView = true;
```
- Added support for zoom-dependent optional content (layers). (DOC-3970, DOC-4108)
- Added the ability to specify a custom progress message during save action.
```javascript
// Example: use your own progress title and message:
viewer.save("sample.pdf", { pages: "[angle:90]0,1-5", progressTitle: "Rotating", progressMessage: "Rotating first page..." }, reload: true);
```
- Added new method resolvePageIndex(): resolves the page index using PDF page reference. (DOC-5214)
```javascript
// Example:
const openAction = (await viewer.viewerPreferences).openAction;
if(openAction && openAction.dest) {
  const pageRef = openAction.dest[0];
  const targetPageIndex = await viewer.resolvePageIndex(pageRef);
}
```
- Added holdToPan action: press and hold the spacebar to temporarily enable the pan tool.
```javascript
// Example 1: disable holdToPan when space is pressed:
viewer.options.shortcuts["32"] = () => {};
// Example 2: bind P to holdToPan action, keep Ctrl+P bound to print action:
viewer.options.shortcuts["P"] = [{ ctrl: true, tool: "print" }, { tool: "holdToPan" }];
```
### Changed
- 'Layers' and 'StructureTree' panels added to the default sidebar layout, 'Articles' panel removed.
  The default set of sidebar panels is now as follows: 'Thumbnails', 'Search', 'Outline', 'Layers', 'StructureTree', 'Attachments'.
```javascript
  // Example: restore the old default layout:
  var viewer = new GcPdfViewer("#root");
  viewer.addArticlesPanel();
  viewer.addThumbnailsPanel();
  viewer.addSearchPanel();
  viewer.addOutlinePanel();
  viewer.addAttachmentsPanel();
```
- [Editor] The printable flag is now set to true for all new annotations.
- Returning model type for the viewer.viewerPreferences property changed: all property names are now camelCased, new properties added: openAction, pageMode, pageLayout.
- Reduced the heights of dialogs' title bars. (DOC-5313) 
- [Editor] Text annotation (sticky note) properties display improved. (DOC-3241)
- hideAnnotationPopups option now allows specifying the types of annotation that won't show popups. (DOC-5283)\
Possible values are: 
['Text',  'Link',  'Line',  'Square',  'Circle',  'Polygon', 'PolyLine',  'Ink',  'Popup',  'FileAttachment', 'Sound',  'Redact', 'Stamp'] or true or 'All' (true and 'All' have the same behavior).
```javascript
// Example 1: hide popups for all annotations:
var viewer = new GcPdfViewer("#root", { hideAnnotationPopups: true });
// Example 2: hide popups for Redact, Circle and Square annotations:
var viewer = new GcPdfViewer("#root", { hideAnnotationPopups: ["Redact", "Circle", "Square"] });
```
- The save() method now allows specifying a range(s) of pages to save, changing page order or duplicating pages. (DOC-3916)
```javascript
// Example 1: save specified pages only:
await viewer.save("test.pdf", { pages: "0, 3, 5-7" });
// Example 2: duplicate and save the first page:
await viewer.save("test.pdf", { pages: "0, 0" }); 
// Example 3: split PDF into two documents:
await viewer.save("test_part1.pdf", { pages: "0-3" });
await viewer.save("test_part2.pdf", { pages: "4-8" });
// Example 4: change page order:
await viewer.save("test_changed_order.pdf", { pages: "3, 2, 1, 0" });
```
- The saveAsImages() method now allows specifying the zoom factor.
```javascript
// Example:
viewer.saveAsImages("sample.pdf", { zoomFactor: 1.5 });
```
### Fixed
- [Editor] In a PDF with different page sizes the 'move to next page' option may not work correctly. (DOC-5217)
- [Editor] A locked stamp annotation can still be edited in GcPdfViewer. (DOC-5216)
- [Editor] Cannot undo changes if the editor layout is not activated. (DOC-5315)
- [Editor] A popup annotation may move unexpectedly. (DOC-5318)
- Cannot navigate between pages using PageUp/PageDown keys in single page view. (DOC-5335)
- The list of layers is not shown correctly in some cases. (DOC-5338)

### See __CHANGELOG.&#8203;md__ for previous release notes.

## Installation

### To install the latest release version:

```sh
npm install @grapecity/gcpdfviewer
```

### To install from the zip archive:

Go to https://www.grapecity.com/download/documents-pdf and follow the directions on that page to get your 30-day evaluation and deployment license key.
The license key will allow you to develop and deploy your application to a test server.
Unzip the viewer distribution files (list below) to an appropriate location accessible from the web page where the viewer will live.

Viewer zip includes the following files:

- README.&#8203;md (this file)
- CHANGELOG.&#8203;md
- gcpdfviewer.js
- gcpdfviewer.worker.js
- package.json
- index.html (sample page)
- Theme files:
  - themes/dark-yellow.css
  - themes/dark-yellow.jscss
  - themes/light-blue.css
  - themes/light-blue.jscss
  - themes/viewer.css
  - themes/viewer.jscss
- Predefined CMap files for Chinese, Japanese, or Korean text output:
  - resource/bcmaps/*.bin
- TypeScript declaration files:
  - typings/*.*

## Documentation

Online documentation is available [here](https://www.grapecity.com/documents-api-pdf/docs/online/grapecitydocumentspdfviewer.html).

## Licensing

GrapeCity Documents PDF Viewer is a commercially licensed product. Please [visit this page](https://www.grapecity.com/licensing/documents-api) for details.

## Getting more help

GrapeCity Documents PDF Viewer is distributed as part of GrapeCity Documents for PDF.
You can ask any questions about the viewer, or report bugs using the
[Documents for PDF public forum](https://www.grapecity.com/forums/documents-pdf).

## More details on using the viewer

### Adding the viewer to an HTML page:

```HTML
<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="utf-8">
    <!-- Limit content scaling to ensure that the viewer zooms correctly on mobile devices: -->
    <meta name="viewport" content="width=device-width, initial-scale=1.0, minimum-scale=1.0, maximum-scale=1.0, user-scalable=no" />
    <meta name="theme-color" content="#000000" />
    <title>GrapeCity Documents PDF Viewer</title>
    <script type="text/javascript" src="lib/gcpdfviewer.js"></script>
    <script>
        function loadPdfViewer(selector) {
            var viewer = new GcPdfViewer(selector,
              {
                /* Specify options here */
                renderInteractiveForms: true
              });
            // Skip the 'report list' panel:
            // viewer.addReportListPanel();
            viewer.addDefaultPanels();
            // Optionally, open a PDF (will only work if this runs from a server):
            viewer.open('HelloWorld.pdf');
            // Change default viewer toolbar:
            viewer.toolbarLayout.viewer.default = ['$navigation', '$split', 'text-selection', 'pan', '$zoom', '$fullscreen',
              'save', 'print', 'rotate', 'view-mode', 'doc-title'];
            viewer.applyToolbarLayout();
        }
    </script>
  </head>
  <body onload="loadPdfViewer('#root')">
    <div id="root"></div>
  </body>
</html>
```

### How to license the viewer:

Set the GcPdfViewer Deployment key to the GcPdfViewer.License property before you create and initialize GcPdfViewer.
This must precede the code that references the js files.

```javascript
  // Add your license
  GcPdfViewer.LicenseKey = 'xxx';
  // Add your code
  const viewer = new GcPdfViewer("#viewer1", { file: 'helloworld.pdf' });
  viewer.addDefaultPanels();
```

### <a id="support_api"></a>Support API

_Support API_ is a server-side library available as NuGet package
[GrapeCity.Documents.Pdf.ViewerSupportApi](https://www.nuget.org/packages/GrapeCity.Documents.Pdf.ViewerSupportApi/).
The full source code of this library together with C# demo projects for ASP.&#8203;NET Core and Web Forms
is included in the __src/__ folder inside the package (the **WEB_FORMS** constant is defined for the Web Forms target).
In your server solution you can either reference the package, or include the source project and reference it instead.

When GcPdfViewer is connected to a Support API server, its editing features are enabled.
The edits done on the client are accumulated, and when the user clicks 'save',
the original PDF and the edits are sent to the server, the edits are applied (using GcPdf),
and the modified PDF is sent back to the client.

To set up a Support API server on your own system and see it in action,
download any of the samples from the [GcPdfViewer demo site](https://www.grapecity.com/documents-api-pdfviewer/demos/)
(e.g. [Edit PDF](https://www.grapecity.com/documents-api-pdfviewer/demos/edit-pdf/purejs)),
and follow the instructions in the readme.&#8203;md included in the downloaded zip.

NOTE that you will need a GrapeCity Documents for PDF Professional license to use Support API in your apps.

### Keyboard shortcuts

#### Viewer mode

- ```Ctrl +``` - zoom in
- ```Ctrl -``` - zoom out
- ```Ctrl 0``` - zoom to 100%
- ```Ctrl 9``` - zoom to window
- ```Ctrl A``` - select all
- ```R``` - rotate clockwise
- ```Shift R``` - rotate counterclockwise
- ```H``` - enable pan tool
- ```S``` - enable selection tool
- ```V``` - enable selection tool
- ```Ctrl O``` - open local PDF
- ```Ctrl F``` - text search
- ```Ctrl P``` - print
- ```Home``` - go to start of line
- ```Ctrl Home``` - go to start of document
- ```Shift Home``` - select  to start of line
- ```Shift Ctrl Home``` - select  to start of document
- ```End``` - go to end of line
- ```Ctrl End``` - go to end of document
- ```Shift End``` - select  to end of line
- ```Shift Ctrl End``` - select to end of document
- ```Left``` - go left
- ```Shift Left``` - select left
- ```Alt Left``` - go back in history
- ```Right``` - go right
- ```Shift Right``` - select right
- ```Alt Right``` - go forward in history
- ```Up``` - go up
- ```Shift Up``` - select up
- ```Down``` - go down
- ```Shift Down``` - select down
- ```PgUp``` - page up
- ```PgDown``` - page down
- ```Shift PgUp``` - select page up
- ```Shift PgDown``` - select page down

#### Editing modes

- ```Delete``` - delete selected annotation/field
- ```Esc``` - unselect annotation/field
- ```Ctrl Z``` - undo
- ```Ctrl Y``` - redo
- ```Ctrl Shift Z``` - redo

### Toolbar items

The default viewer toolbar items layout (items starting with '$' are inherited from the base viewer, other items are PDF viewer specific.):

```
['open', '$navigation', '$split', 'text-selection', 'pan', '$zoom', '$fullscreen', 'rotate', 'view-mode', 'theme-change', 'download', 'print', 'save', 'hide-annotations', 'doc-title', 'doc-properties', 'about']
```

The full list of the PDF-viewer specific toolbar items:

```
'text-selection', 'pan', 'open', 'save', 'download', 'print', 'rotate', 'theme-change', 'doc-title', 'view-mode', 'single-page', 'continuous-view'
```

The full list of base viewer toolbar items:

```
'$navigation' '$refresh', '$history', '$mousemode', '$zoom', '$fullscreen', '$split'
```

You can get default base viewer items by using the getDefaultToolbarItems() method, e.g.:

```javascript
const toolbar: Toolbar = viewer.toolbar;
let buttons = toolbar.getDefaultToolbarItems();
buttons = buttons.filter(b => b !== '$refresh');
```

To specify a custom set of toolbar items, use the toolbarLayout property and applyToolbarLayout() method, e.g.:

```javascript
viewer.toolbarLayout.viewer = {
  default: ["$navigation", 'open', '$split', 'doc-title'],
  fullscreen: ['$fullscreen', '$navigation'],
  mobile: ["$navigation", 'doc-title']
};
viewer.toolbarLayout.annotationEditor = {
  default: ['edit-select', 'save', '$split', 'edit-text'],
  fullscreen: ['$fullscreen', 'edit-select', 'save', '$split', 'edit-text'],
  mobile: ['$navigation']
};
viewer.toolbarLayout.formEditor = {
  default: ['edit-select-field', 'save', '$split', 'edit-widget-tx-field', 'edit-widget-tx-password'],
  fullscreen: ['$fullscreen', 'edit-select-field', 'save', '$split', 'edit-widget-tx-field', 'edit-widget-tx-password'],
  mobile: ['$navigation']
};
viewer.applyToolbarLayout();
```

Here is an example of how to create your own custom toolbar button:

```javascript
const toolbar: Toolbar = viewer.toolbar;
toolbar.addItem({
  key: 'my-custom-button',
  iconCssClass: 'mdi mdi-bike',
  title: 'Custom button',
  enabled: false,
  action: () => {
    alert("Custom toolbar button clicked");
  },
  onUpdate: (args) => ({ enabled: viewer.hasDocument }),
});
viewer.toolbarLayout.viewer.default =  ['$navigation', 'my-custom-button'];
viewer.applyToolbarLayout();
```

### Using the viewer in frameworks such as React, Preact, Angular etc.

Add a reference to the viewer script:
```HTML
<body>
  <script type="text/javascript" src="/lib/gcpdfviewer/gcpdfviewer.js"></script>
  ...
```

Add the placeholder to your App template, e.g.:
```HTML
<section id="pdf"></section>
```

Render the viewer:
```javascript
let viewer = new GcPdfViewer('section#pdf');
viewer.addDefaultPanels();
```

---

# <a id="japanese"></a>GrapeCity PDF ビューワ（日本語版）

__GrapeCity PDF ビューワ__ (__GcPdfViewer__) は、主要なブラウザで動作する、JavaScriptベースのPDFビューワおよびエディタです。Windows、MAC、Linux、iOS、Androidなどのデバイス上でPDFドキュメントを表示 (または編集 - 下記の _サポートAPI_ を参照) するためのクロスプラットフォームソリューションとして使用することができます。
なお、本PDFビューワは、クロスプラットフォーム環境にてPDFを作成・編集できるAPIライブラリ [DioDocs for PDF](https://www.grapecity.co.jp/developer/diodocs/pdf) に付属する製品となります。

[_サポートAPI_](#support_api_ja) は、サーバーサイドのNuGetパッケージ
「[GrapeCity.DioDocs.Pdf.ViewerSupportApi.ja](https://www.nuget.org/packages/GrapeCity.DioDocs.Pdf.ViewerSupportApi.ja/)」で、
GcPdfに接続するサーバーをASP.&#8203;NET CoreまたはWeb Formsにて簡単に構築して、
PDFビューワにエディタ機能を追加できるようにするものです。
_サポートAPI_ サーバーに接続すると、PDFビューワをPDFエディタとして使用し、
PDFの新規作成、既存のPDFの編集、PDFフォームの入力や保存、機密コンテンツの削除（墨消し）、
注釈／フォームフィールドの追加／編集などを行うことができるようになります。

PDFビューワのクライアントAPIの詳細については、[こちら](https://docs.grapecity.com/help/diodocs/pdfviewer/)をご参照ください。

製品の特徴：
- Edge、Chrome、FireFox、Opera、Safariを含むすべての最新ブラウザで動作します
- _サポートAPI_ 経由で _GcPdf_  に接続した場合、以下の機能を提供します：
  - カスタマイズ可能でモバイルフレンドリーなフォームフィラー
  - リアルタイムなコラボレーションモード
  - 注釈とフォームの編集
  - セカンドツールバーを使用したクイック編集
  - PDFコンテンツの墨消し
  - 署名の検証
  - その他の編集機能
- React、Preact、Angular などのフレームワークで動作します
- フォームの入力に対応：記入済みフォームの印刷や、フォームデータのサーバへの送信にも対応しています
- XFA（XML Forms Architecture）フォームに対応しています
- 縦書きテキストや右横書きテキストを含む、テキストの選択/コピーのためのキャレットを提供します
- サムネイル、テキスト検索、ブックマーク、添付ファイル、アーティクル、レイヤー、構造ツリーのパネルが含まれます
- ローカルディスクからPDFファイルを開くことができます
- テキスト、フリーテキスト、リッチテキストなどの注釈に対応しています
- 墨消し注釈（APストリームを含む）に対応し、墨消しの表示・非表示が可能です
- 音声注釈に対応しています
- 文書の回転や、回転させた文書の印刷が可能です
- アーティクルのスレッドナビゲーションに対応しています
- 添付ファイル（添付ファイル注釈と文書レベルの添付ファイルの両方）に完全に対応しています
- 複数のテーマがすでに備わっており、さらに新しいカスタムテーマを追加することも可能です
- 外部のCMapに対応しています
- その他、様々な機能を提供しています

## リリースノート
日本語版として動作を保証しているのは、リリースノートに記載しているバージョンのみとなります。

## [4.0.1] - 2023年2月8日
### 追加
- テキストマークアップ注釈（ハイライト、下線、波状下線、取り消し線）に対応しました。
- ツールバーとコンテキストメニューに、テキストマークアップ注釈（ハイライト、下線、波状下線、取り消し線）のための機能を追加しました。
```javascript
// テキストマークアップ注釈のコンテキストメニューを表示しないようにします。
var viewer = new GcPdfViewer(selector, { 
	textMarkupContextMenu: false
});
// テキストマークアップ注釈のコンテキストメニューに、色の選択メニューを表示しないようにします。
var viewer = new GcPdfViewer(selector, { 
  textMarkupContextMenu: { colors: [] }
});
// テキストマークアップ注釈のコンテキストメニューにて選択できる色を、「赤」と「黒」に変更します。
var viewer = new GcPdfViewer(selector, { 
	textMarkupContextMenu: { colors: [{value: "#ff0000", displayName: "赤"}, {value: "#000000", displayName: "黒"}] }
});
```
- PDF のページを画像として保存する機能を追加しました。
- saveAsImages メソッドを追加しました。現在の PDF ドキュメントのページを PNG 画像として保存し、それらの画像を zip に圧縮してダウンロードします。
 ```javascript
 // 例：
 viewer.saveAsImages('test.zip');
 ```
- ツールバーに「現在のドキュメントを画像として保存」ボタンを追加しました。
- customIcons プロパティにて、PDF ビューワーが使用する SVG 形式のアイコンを変更できるようになりました。
```javascript
// 例：
var viewer = new GcPdfViewer("#root", { 
  customIcons: {
    'open': '<svg xmlns="http://www.w3.org/2000/svg" version="1.1" width="24" height="24" viewBox="0 0 24 24"><path d="M9.516 14.016q1.875 0 3.188-1.313t1.313-3.188-1.313-3.188-3.188-1.313-3.188 1.313-1.313 3.188 1.313 3.188 3.188 1.313zM15.516 14.016l4.969 4.969-1.5 1.5-4.969-4.969v-0.797l-0.281-0.281q-1.781 1.547-4.219 1.547-2.719 0-4.617-1.875t-1.898-4.594 1.898-4.617 4.617-1.898 4.594 1.898 1.875 4.617q0 0.984-0.469 2.227t-1.078 1.992l0.281 0.281h0.797z"></path></svg>',
    'search': '<svg xmlns="http://www.w3.org/2000/svg" version="1.1" width="24" height="24" viewBox="0 0 24 24"><path d="M9.516 14.016q1.875 0 3.188-1.313t1.313-3.188-1.313-3.188-3.188-1.313-3.188 1.313-1.313 3.188 1.313 3.188 3.188 1.313zM15.516 14.016l4.969 4.969-1.5 1.5-4.969-4.969v-0.797l-0.281-0.281q-1.781 1.547-4.219 1.547-2.719 0-4.617-1.875t-1.898-4.594 1.898-4.617 4.617-1.898 4.594 1.898 1.875 4.617q0 0.984-0.469 2.227t-1.078 1.992l0.281 0.281h0.797z"></path></svg>'
  }
});
```
- マウスを使用して署名を配置し、ページ間でドラッグ移動できるようになりました。\
  また、signTool.location プロパティが指定されていない場合、署名ツールにより追加される署名を任意の場所（別ページなど）にドラッグ移動できるようになりました。
```javascript
// 以前の動作を維持する方法：
// 例1：
var viewer = new GcPdfViewer("#root", { 
	signTool: {
		location: "BottomRight"
	}
});
// 例2：
viewer.options.signTool = { location: "BottomRight" };
```
- スタンプ注釈をページ間でドラッグ移動できるようになりました。
- Undo ストアの現在の状態を取得できる undoStore プロパティを追加しました。\
  なお、このプロパティは読み取り専用であり、プログラムによるコレクション要素の変更はできないことに注意してください。\
  undoState を変更するには、以下の Undo API を使用します。\
  利用可能な Undo API のプロパティ：hasUndoChanges, hasRedoChanges, undoIndex, undoCount\
  利用可能な Undo API のメソッド：undoChanges(), redoChanges()
```javascript
// 例：
viewer.eventBus.on("undostorechanged", function() {
    console.log("undoState が変更されました。", viewer.undoState);
});
// または：
viewer.eventBus.on("documentchanged", function() {
    console.log("ドキュメントが変更されました。", viewer.undoState);
});
```
- 音声注釈に「耳」アイコンを追加しました。
- 増分更新またはリニアライズして PDF を保存できるようになりました。
```javascript
// 例：IncrementalUpdate モードを使用して、増分更新でドキュメントを保存します。
viewer.save("test.pdf", { saveMode: "IncrementalUpdate" });
```
```javascript
// 例：Linearized モードを使用して、リニアライズしてドキュメントを保存します。
viewer.save("test.pdf", {saveMode: "Linearized"});
```
- 更新した PDF ドキュメントをローカルにダウンロードせずにビューワに再表示（リロード）する機能を追加しました。
```javascript
// 例：ドキュメントを保存し、その保存したドキュメントをビューワに再表示します。
await viewer.save("test.pdf", { reload: true });
```
- 電子署名の API を追加しました。
```javascript
// 例：ドキュメントに署名して保存します。
viewer.save("test.pdf", { sign: { signatureField: "field1", signerName: "John Doe" } } );
```
　　サーバー側での署名の例：
```csharp
// 例：.PFX 証明書で PDF に電子署名を行います。
public void Configuration(IAppBuilder app) {
  GcPdfViewerController.Settings.Sign += _OnSign;
  // ... 
}
private void _OnSign(object sender, SignEventArgs e)
{
    var signatureProperties = e.SignatureProperties;
    X509Certificate2 certificate = new X509Certificate2(System.IO.File.ReadAllBytes("certificate.pfx"), "password", 
      X509KeyStorageFlags.MachineKeySet | X509KeyStorageFlags.PersistKeySet | X509KeyStorageFlags.Exportable);
    signatureProperties.SignatureBuilder = new Pkcs7SignatureBuilder()
    {
       CertificateChain = new X509Certificate2[] { certificate },
       HashAlgorithm = Security.OID.HashAlgorithms.SHA512,
       Format = Pkcs7SignatureBuilder.SignatureFormat.adbe_pkcs7_detached
    };
}
```
- 署名情報を取得する API を追加しました。
```javascript
 // 例：現在のドキュメントに署名があるかどうかを確認し、署名に関する情報を表示します。
 var viewer = GcPdfViewer.findControl("#root");
 const signatureInfo = await viewer.getSignatureInfo();
 if(signatureInfo.signed) {
   const signatureValue = signatureInfo.signedByFields[0].signatureValue;
   const signerName = signatureValue.name;
   const location = signatureValue.location;
   const signDate = viewer.pdfStringToDate(signatureValue.modificationDate);
   alert("このドキュメントには署名があります。署名者：" + signerName + "、場所：" + location + "、署名日：" + signDate.toString());
 } else {
   alert("このドキュメントには署名はありません。");
 }
```
- プロパティパネルにて、線注釈のサイズを変更できるようになりました。
- addAnnotation／updateAnnotation メソッドのパラメータをオーバーライドする機能を追加しました。
```javascript
// 例：
const pageIndex = 0;
// pageIndex パラメータを持たないオーバーライドされたメソッドを使用します。
annotation.pageIndex = pageIndex;
await viewer.addAnnotation(annotation);
await viewer.updateAnnotation(annotation);
// pageIndex パラメータを持つメソッドを使用します。
await viewer.addAnnotation(pageIndex, annotation);
await viewer.updateAnnotation(pageIndex, annotation);
```

### 変更
- ページビューのデフォルトの余白を変更（左右の余白を追加）しました。
- アーティクルのスレッドコンテンツの枠線ハイライトを削除しました。
```javascript
// 以下のコードを使用すると、以前のアーティクルのスタイルに戻すことができます。
GcPdfViewer.findControl("#root").addViewAreaStyle(".gc-thread-bead { box-shadow: rgba(255, 232, 0, 0.27) 1px 1px 1px 1px;}");
```
- 付箋／音声／添付ファイル注釈のアイコンを Acrobat Reader のアイコンと同様になるよう変更しました。
- 付箋／音声／添付ファイル注釈は、サイズ変更できなくなりました。注釈のサイズは、選択されたアイコンの種類に依存します。
- 「テキスト」のクイック編集ツールにテキストマークアップ注釈のボタンが追加されました。
- 署名ツールのデフォルトの動作が変更になりました。\
  signTool.location プロパティが指定されていない場合、署名ツールにより追加される署名をマウスを使用して任意の場所に移動できるようになります。

### 不具合の修正
- [[5630529354255](https://developer-tools.zendesk.com/hc/ja/articles/5630529354255)] Windowsのタッチデバイス上で「手のひらツール」が正しく動作しない
- [[5650428545551](https://developer-tools.zendesk.com/hc/ja/articles/5650428545551)] スタンプ注釈を一度回転させると、元の角度に戻してもスタンプ画像のサイズを変更できなくなる
- [[5650450427151](https://developer-tools.zendesk.com/hc/ja/articles/5650450427151)] 回転されたページに署名を追加すると正しい向きにならない
- [[5658427842831](https://developer-tools.zendesk.com/hc/ja/articles/5658427842831)] ２バイト文字を含む名前のPDFを開いて保存すると、２バイト文字を除いた名前になる
- [[5663388780175](https://developer-tools.zendesk.com/hc/ja/articles/5663388780175)] 添付ファイルパネルに添付ファイル注釈以外のファイルが表示される
- [[5663407932431](https://developer-tools.zendesk.com/hc/ja/articles/5663407932431)] 線注釈を水平または垂直にすると表示されなくなる
- [[5728182304271](https://developer-tools.zendesk.com/hc/ja/articles/5728182304271)] 適用済みの墨消しの上に他の注釈を配置したとき、その注釈がPDFに保存されない
- [[5728185724175](https://developer-tools.zendesk.com/hc/ja/articles/5728185724175)] 墨消しの上に配置された注釈をコンテンツに変換すると、ドキュメント保存時にエラーが発生する
- [[6116629704079](https://developer-tools.zendesk.com/hc/ja/articles/6116629704079)] 単一ページ表示状態でブラウザの表示倍率を変更すると、スクロール位置が自動で先頭に戻る
- [[6160006702095](https://developer-tools.zendesk.com/hc/ja/articles/6160006702095)] 特定のPDFにスタンプ注釈を追加してコンテンツに変換すると、保存時にエラーが発生する
- [[6160010153487](https://developer-tools.zendesk.com/hc/ja/articles/6160010153487)] 特定のPDFについて、注釈を正しく追加できない

##### 過去のリリースノートについては、 __CHANGELOG-JP.&#8203;md__ をご参照ください。

## 製品デモ
- [PDFビューワの製品デモ](https://demo.grapecity.com/diodocs/pdfviewer/demos/) では、[_サポートAPI_](#support_api_ja) を使用する編集機能を含め、
  PDFビューワの様々な機能を紹介しています。
  このデモでは、クライアント側のコードを変更し、どのように反映されるか確認することもできます。
- [DioDocs for PDFの製品デモ](https://demo.grapecity.com/diodocs/pdf/) 内のすべてのデモでは、PDFビューワを使用してPDFを表示しています。

## インストール
```sh
npm install @grapecity/gcpdfviewer
```

## 製品ヘルプ
製品ヘルプは[こちら](https://docs.grapecity.com/help/diodocs/pdf/#grapecitydocumentspdfviewer.html)からご覧いただけます。

## ライセンス
GrapeCity PDF ビューワのご利用にはライセンスが必要となります。詳しくはWebサイトの[ライセンス手続き](https://www.grapecity.co.jp/developer/support/license#docapi)ページをご覧ください。

## 製品情報
GrapeCity PDF ビューワは、DioDocs for PDF の一部として提供しております。製品に関する詳細な情報については、Webサイトの[製品情報](https://www.grapecity.co.jp/developer/diodocs/pdf)ページをご参照ください。

## PDFビューワの詳細な使用方法
### HTMLページへのPDFビューワの追加
```HTML
<!DOCTYPE html>
<html lang="ja">
<head>
    <meta charset="utf-8">
    <!--コンテンツの拡大縮小を制限して、モバイルデバイスでビューワが正しくズームできるようにします。-->
    <meta name="viewport" content="width=device-width, initial-scale=1.0, minimum-scale=1.0, maximum-scale=1.0, user-scalable=no" />
    <meta name="theme-color" content="#000000" />
    <title>GrapeCity PDF ビューワ</title>
    <script type="text/javascript" src="lib/node_modules/@grapecity/gcpdfviewer/gcpdfviewer.js"></script>
    <script>
        function loadPdfViewer(selector) {
            var viewer = new GcPdfViewer(selector,
                {
                    /* ここでオプションを指定します */
                    renderInteractiveForms: true
                });
            viewer.addDefaultPanels();
            // 必要に応じてPDFを開きます（サーバーから実行時にのみ機能）
            viewer.open('HelloWorld.pdf');
            // デフォルトのビューワのツールバーを変更します
            viewer.toolbarLayout.viewer.default = ['$navigation', '$split', 'text-selection', 'pan', '$zoom', '$fullscreen',
                'save', 'print', 'rotate', 'view-mode', 'doc-title'];
            viewer.applyToolbarLayout();
        }
    </script>
</head>
<body onload="loadPdfViewer('#root')">
    <div id="root"></div>
</body>
</html>
```

### PDFビューワへのライセンスの適用
GcPdfViewer のインスタンスを作成して初期化する前に、PDFビューワのライセンスキーを GcPdfViewer.License プロパティに設定します。
なお、これは jsファイルを参照するコードの前に記述する必要があります。

```javascript
  // ご自身のライセンスを追加してください
  GcPdfViewer.LicenseKey = 'xxx';
  // 適宜コードを追加してください
  const viewer = new GcPdfViewer('#root', { file: 'helloworld.pdf' });
  viewer.addDefaultPanels();
```

### PDFビューワの検索オプション
PDFビューワには、以下のようなテキストの検索オプションが用意されていますが、**英語のみの対応**となっており、日本語は正しく検索できない場合があります。
- 大/小文字を区別
- 単語単位で検索
- 単語の先頭を検索
- 単語の末尾を検索
- ワイルドカード
- 近接

上記検索オプションを非表示にしたい場合は、ページの`<head>`部分に以下のとおりCSSスタイルを追加してください。
```HTML
<style>
    .gc-viewer .search .search__query-params > label:nth-child(1),
    .gc-viewer .search .search__query-params > label:nth-child(2),
    .gc-viewer .search .search__query-params > label:nth-child(3),
    .gc-viewer .search .search__query-params > label:nth-child(4),
    .gc-viewer .search .search__query-params > label:nth-child(5),
    .gc-viewer .search .search__query-params > label:nth-child(6) {
        display: none;
    }
</style>
```

### <a id="support_api_ja"></a>サポートAPI
_サポートAPI_ は、NuGetパッケージ
「[GrapeCity.DioDocs.Pdf.ViewerSupportApi.ja](https://www.nuget.org/packages/GrapeCity.DioDocs.Pdf.ViewerSupportApi.ja/)」として
提供されるサーバーサイドのライブラリです。
このライブラリの完全なソースコードは、ASP.&#8203;NET CoreおよびWeb FormsのC＃デモプロジェクトとともに、
パッケージ内の「 __src/__ 」フォルダに含まれています（Web Formsのために **WEB_FORMS** 定数が定義されています）。
サーバーの構築方法として、パッケージを参照するか、または代わりにソースプロジェクトを含めてそれを参照するか、選ぶことができます。

PDFビューワがサポートAPIサーバーに接続されている場合、編集機能が有効になります。
クライアント上で行われた編集は蓄積され、ユーザーが「保存」をクリックすると、
元のPDFと編集内容がサーバーに送られて、編集が（GcPdfを使って）適用され、
変更されたPDFがクライアントに送り返されます。

サポートAPIサーバーをセットアップして動作を確認するには、
[PDFビューワの製品デモ](https://demo.grapecity.com/diodocs/pdfviewer/demos/)から任意のサンプルをダウンロードし
（例：[PDFの編集](https://demo.grapecity.com/diodocs/pdfviewer/demos/edit-pdf/purejs)）、
ダウンロードしたzipに含まれるreadme.&#8203;mdの説明をご参照ください。

なお、サポートAPIを使用するには、有効なDioDocs for PDFのライセンスが必要です。
ライセンスは、GcPdfDocumentのSetLicenseKey静的メソッドにて設定します。以下は、ASP.&#8203;NET Coreプロジェクトの「Startup.cs」にてDioDocs for PDFライセンスをサポートAPIに適用する例です。
```C#
public class Startup
{
    static Startup()
    {
        // 略
        GcPdfDocument.SetLicenseKey("ライセンスキー");
    }
    // 略
}
```

### キーボードのショートカット
#### 表示モード
- ```Ctrl +``` - 拡大
- ```Ctrl -``` - 縮小
- ```Ctrl 0``` - 100％に拡大
- ```Ctrl 9``` - ページ幅に合わせて拡大
- ```Ctrl A``` - すべて選択
- ```R``` - ドキュメントを右回りに回転
- ```Shift R``` - ドキュメントを左回りに回転
- ```H``` - 手のひらツールを有効化
- ```S``` - 選択ツールを有効化
- ```Ctrl O``` - PDFファイルを開く
- ```Ctrl F``` - テキスト検索
- ```Ctrl P``` - 印刷
- ```Home``` - 行頭に移動
- ```Ctrl Home``` - ドキュメントの先頭に移動
- ```Shift Home``` - 行頭まで選択
- ```Shift Ctrl Home``` - ドキュメントの先頭まで選択
- ```End``` - 行末に移動
- ```Ctrl End``` - ドキュメントの末尾に移動
- ```Shift End``` - 行末まで選択
- ```Shift Ctrl End``` - ドキュメントの末尾まで選択
- ```Left``` - 左に移動
- ```Shift Left``` - 左を選択
- ```Alt Left``` - 前の履歴に戻る
- ```Right``` - 右に移動
- ```Shift Right``` - 右を選択
- ```Alt Right``` - 次の履歴に進む
- ```Up``` - 上に移動
- ```Shift Up``` - 上まで選択
- ```Down``` - 下に移動
- ```Shift Down``` - 下まで選択
- ```PgUp``` - 前のページに移動
- ```PgDown``` - 次のページに移動
- ```Shift PgUp``` - 前のページまで選択
- ```Shift PgDown``` - 次のページまで選択

#### 編集モード
- ```Delete``` - 選択した注釈/フィールドを削除
- ```Esc``` - 注釈/フィールドの選択を解除
- ```Ctrl Z``` - 元に戻す
- ```Ctrl Y``` - やり直す
- ```Ctrl Shift Z``` - やり直す

### ツールバーの項目
PDFビューワのデフォルトのツールバーレイアウトは以下のとおりです。（'$'で始まる項目は、ビューワの標準として備わっているものであり、それ以外の項目はPDFビューワ固有のものになります。）
```
['open', '$navigation', '$split', 'text-selection', 'pan', '$zoom', '$fullscreen', 'rotate', 'view-mode', 'theme-change', 'download', 'print', 'save', 'hide-annotations', 'doc-title', 'doc-properties', 'about']
```

PDFビューワ固有のツールバー項目は以下のとおりです。
```
'text-selection', 'pan', 'open', 'save', 'download', 'print', 'rotate', 'theme-change', 'doc-title', 'view-mode', 'single-page', 'continuous-view'
```

ビューワの標準のツールバー項目は以下のとおりです。
```
'$navigation' '$refresh', '$history', '$mousemode', '$zoom', '$fullscreen', '$split'
```

以下のように、getDefaultToolbarItems() メソッドを使用することで、デフォルトのツールバー項目を取得することができます。
```javascript
const toolbar: Toolbar = viewer.toolbar;
let buttons = toolbar.getDefaultToolbarItems();
buttons = buttons.filter(b => b !== '$refresh');
```

ツールバー項目をカスタマイズするには、以下のように toolbarLayout プロパティと applyToolbarLayout() メソッドを使用します。
```javascript
viewer.toolbarLayout.viewer = {
  default: ["$navigation", 'open', '$split', 'doc-title'],
  fullscreen: ['$fullscreen', '$navigation'],
  mobile: ["$navigation", 'doc-title']
};
viewer.toolbarLayout.annotationEditor = {
  default: ['edit-select', 'save', '$split', 'edit-text'],
  fullscreen: ['$fullscreen', 'edit-select', 'save', '$split', 'edit-text'],
  mobile: ['$navigation']
};
viewer.toolbarLayout.formEditor = {
  default: ['edit-select-field', 'save', '$split', 'edit-widget-tx-field', 'edit-widget-tx-password'],
  fullscreen: ['$fullscreen', 'edit-select-field', 'save', '$split', 'edit-widget-tx-field', 'edit-widget-tx-password'],
  mobile: ['$navigation']
};
viewer.applyToolbarLayout();
```

以下のように、独自のカスタムツールバーボタンを作成することもできます。
```javascript
const toolbar: Toolbar = viewer.toolbar;
toolbar.addItem({
  key: 'my-custom-button',
  iconCssClass: 'mdi mdi-bike',
  title: 'カスタムボタン',
  enabled: false,
  action: () => {
    alert("カスタムツールバーボタンがクリックされました");
  },
  onUpdate: (args) => ({ enabled: viewer.hasDocument }),
});
viewer.toolbarLayout.viewer.default =  ['$navigation', 'my-custom-button'];
viewer.applyToolbarLayout();
```

### React、Preact、AngularなどのフレームワークでのPDFビューワの使用
PDFビューワのスクリプトへの参照を追加します。
```HTML
<body>
  <script type="text/javascript" src="/lib/gcpdfviewer/gcpdfviewer.js"></script>
  ...
```

アプリのテンプレートに以下のようにプレースホルダを追加します。
```HTML
<section id="pdf"></section>
```

PDFビューワをレンダリングします。
```javascript
let viewer = new GcPdfViewer('section#pdf');
viewer.addDefaultPanels();
```

---
_The End._
